import pulumi
import base64
from pulumi import Output
from pulumi_aws import ec2
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, Mapping, List

from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION

logger = Logger("ec2")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class InstanceSettings:
    ami_name: Optional[str] = None
    ami_id: Optional[str] = None
    instance_type: Optional[str] = None
    subnet_id: Optional[str] = None
    key_name: Optional[str] = None
    associate_public_ip_address: Optional[bool] = False
    security_groups: Optional[List[str | Output[str]]] = None
    iam_instance_profile: Optional[str] = None
    instance_metadata_tags: Optional[str] = None
    associate_elastic_ip: Optional[bool] = None
    user_data: Optional[str] = None


class Instance:
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[InstanceSettings] = InstanceSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> ec2.InstanceArgs:

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        logger.info(f"creating ec2 instance {resource_name}")

        if settings.ami_name:
            ami = ec2.get_ami(
                most_recent=True,
                filters=[
                    ec2.GetAmiFilterArgs(
                        name="name",
                        values=[settings.ami_name],
                    )
                ],
            )

        instance_args = ec2.InstanceArgs(
            ami=settings.ami_id if settings.ami_id else ami.id,
            instance_type=settings.instance_type,
            subnet_id=settings.subnet_id,
            key_name=settings.key_name,
            associate_public_ip_address=settings.associate_public_ip_address,
            vpc_security_group_ids=settings.security_groups,
            iam_instance_profile=settings.iam_instance_profile,
            metadata_options=ec2.InstanceMetadataOptionsArgs(instance_metadata_tags=settings.instance_metadata_tags),
            user_data_base64=None if not settings.user_data else base64.b64encode(settings.user_data.encode()).decode(),
            tags=merged_tags,
        )

        instance_out = ec2.Instance(resource_name=resource_name, args=instance_args, opts=opts)

        if settings.associate_elastic_ip:
            ec2.Eip(resource_name=f"{resource_name}-elastic-ip", instance=instance_out.id, tags=merged_tags)

        pulumi.export(
            f"ec2-instance---{resource_name}",
            {
                "id": instance_out.id,
                "arn": instance_out.arn,
                "public_ip": instance_out.public_ip,
                "private_ip": instance_out.private_ip,
                "ami_id": settings.ami_id if settings.ami_id else ami.id,
                "ami_name": settings.ami_name if settings.ami_name else "",
            },
        )

        return instance_out

    @staticmethod
    def import_(stack: str, resource_name: str) -> ec2.Instance:
        stack_ref = StackRegistry.get(stack_name=stack)
        instance = stack_ref.get_output(name=f"ec2-instance---{resource_name}")
        return ec2.Instance.get(resource_name=resource_name, id=instance["id"])
