import pulumi

from pulumi_aws import eks
from pulumi.output import Output

from dataclasses import field
from pydantic.dataclasses import dataclass, ConfigDict
from typing import List, Mapping, Optional, Literal

from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION, AMI_TYPES

logger = Logger("k8s")


@dataclass
class ScalingConfig:
    desired_size: int
    max_size: int
    min_size: int


@dataclass
class NodeGroupTaint:
    effect: Literal["NO_SCHEDULE", "NO_EXECUTE", "PREFER_NO_SCHEDULE"]
    key: str
    value: Optional[str]


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NodeGroupSettings:
    ami_type: Optional[AMI_TYPES] = None
    disk_size: Optional[int] = None
    force_update_version: Optional[bool] = False
    labels: Optional[Mapping[str, str | Output[str]]] = None
    instance_types: Optional[List[str]] = field(default_factory=lambda: ["t3.medium"])
    release_version: Optional[str] = None
    taints: Optional[List[NodeGroupTaint]] = None
    version: Optional[str] = None


class NodeGroup:
    def __new__(
        cls,
        resource_name: str,
        cluster_name: str,
        node_role_arn: str,
        subnet_ids: List[str],
        scaling_config: ScalingConfig,
        capacity_type: Literal["ON_DEMAND", "SPOT"],
        launch_template_version: Optional[str] = None,
        launch_template_id: Optional[str] = None,
        settings: Optional[NodeGroupSettings] = NodeGroupSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> eks.NodeGroup:
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        scaling = eks.NodeGroupScalingConfigArgs(
            desired_size=scaling_config.desired_size,
            max_size=scaling_config.max_size,
            min_size=scaling_config.min_size,
        )

        taint_args = None
        if settings.taints:
            taint_args = [
                eks.NodeGroupTaintArgs(
                    effect=taint.effect, key=taint.key, value=taint.value
                )
                for taint in settings.taints
            ]

        nodegroup = eks.NodeGroup(
            resource_name,
            cluster_name=cluster_name,
            node_role_arn=node_role_arn,
            subnet_ids=subnet_ids,
            scaling_config=scaling,
            node_group_name=resource_name,
            ami_type=settings.ami_type,
            capacity_type=capacity_type,
            disk_size=settings.disk_size,
            force_update_version=settings.force_update_version,
            instance_types=settings.instance_types,
            labels=settings.labels,
            launch_template=eks.NodeGroupLaunchTemplateArgs(
                version=launch_template_version, id=launch_template_id
            ),
            taints=taint_args,
            tags=merged_tags,
            version=settings.version,
            opts=opts,
        )

        pulumi.export(
            f"eks-nodegroup---{resource_name}",
            {
                "id": nodegroup.id,
                "arn": nodegroup.arn,
                "resources": nodegroup.resources,
                "status": nodegroup.status,
            },
        )

        return nodegroup

    @staticmethod
    def import_(stack: str, resource_name: str) -> eks.NodeGroup:
        stack_ref = StackRegistry.get(stack_name=stack)
        nodegroup = stack_ref.get_output(name=f"eks-nodegroup---{resource_name}")
        return eks.NodeGroup.get(resource_name=resource_name, id=nodegroup["id"])
