from typing import List, Optional
import pulumi
import pulumi_aws
from theia.aws.network.network import Network
from theia.aws.network.subnet import Subnet
from theia.utils.errors import TheiaConfigurationError

from theia.aws.security.sg import SecurityGroup, SecurityGroupSettings

class DocDB:
    def __new__(cls,
            resource_name,
            instance_class,
            engine, username,
            password_secret,
            sg_id: Optional[List[str | pulumi.Output[str]]],
            network: pulumi.Output[Network],
            allow_access_to_ec2_id: Optional[str | pulumi.Output[str]]=None,
            allow_access_to_eks_cluster_name: Optional[str | pulumi.Output[str]]=None
        ) -> pulumi_aws.docdb.Cluster:
        
        subnet_ids = [subnet.id for subnet in network.subnets["private"]]
        subnet_group = pulumi_aws.docdb.SubnetGroup(resource_name=f"{resource_name}-docdb-subnet-group",
            subnet_ids=subnet_ids,
            name=f"{resource_name}-docdb-subnet-group",
            opts=pulumi.ResourceOptions(depends_on=[network])
        )

        if (not sg_id):
            security_groups = []
            if allow_access_to_ec2_id:
                security_groups = pulumi.Output.all(allow_access_to_ec2_id).apply(lambda instance_id: pulumi_aws.ec2.get_instance(instance_id=instance_id).security_groups)
            if allow_access_to_eks_cluster_name:
                existing_eks_cluster = pulumi_aws.eks.get_cluster(name=allow_access_to_eks_cluster_name)
                security_groups.append(pulumi.Output.all(existing_eks_cluster.vpc_config["securityGroupIds"]).apply(lambda sg_ids: pulumi_aws.ec2.get_security_groups(ids=sg_ids)))

            args = [
                SecurityGroupSettings(
                    sg_type="egress", traffic_type="All TCP", cidr_blocks=["0.0.0.0/0"]
                ),
                SecurityGroupSettings(
                    sg_type="ingress", from_port=27017, to_port=27017, security_groups=security_groups,traffic_type="Custom", protocol='tcp'
                )
            ]
            sg = SecurityGroup(
                f"{resource_name}-docdb-sg",
                network.vpc.id,
                args,
                "sg created from Theia"
            )
        cluster = pulumi_aws.docdb.Cluster(resource_name=resource_name,
            cluster_identifier=resource_name,
            engine=engine,
            db_subnet_group_name=f"{resource_name}-docdb-subnet-group",
            master_username=username,
            master_password=pulumi.Output.secret(password_secret),
            skip_final_snapshot=True,
            vpc_security_group_ids=[sg.id] if (not sg_id) else sg_id,
            opts=pulumi.ResourceOptions(depends_on=[subnet_group])
        )

        _ints = pulumi_aws.docdb.ClusterInstance(
            f"{resource_name}-clusterinstances-0",
            identifier=f"{resource_name}-docdb-cluster-demo-0",
            cluster_identifier=resource_name,
            instance_class=instance_class,
            opts=pulumi.ResourceOptions(depends_on=[cluster])
        )
        pulumi.export(
            f"DocumentDB---{resource_name}", {"id": cluster.id, "arn": cluster.arn, "endpoint": cluster.endpoint, "ClusterResourceId": cluster.cluster_resource_id}
        )
        
        return cluster
