import pulumi
from pulumi_aws import dynamodb
from pulumi.output import Output
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, List, Mapping, Literal, Sequence
from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.utils.errors import TheiaConfigurationError


logger = Logger("dynamodb")

@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class TableSettings:
    billing_mode: Optional[
        Literal["PROVISIONED", "PAY_PER_REQUEST"]
    ] = "PROVISIONED"
    deletion_protection_enabled: Optional[bool] = None
    hash_key: Optional[str] = None
    range_key: Optional[str] = None
    read_capacity: Optional[int] = None
    stream_view_type: Optional[Literal["KEYS_ONLY","NEW_IMAGE","OLD_IMAGE","NEW_AND_OLD_IMAGES"]] = None
    table_class: Optional[
        Literal["STANDARD","STANDARD_INFREQUENT_ACCESS"]
    ] = None
    write_capacity: Optional[int] = None
    
    def __post_init__(self):
        if (self.billing_mode == "PROVISIONED" and not self.write_capacity):
            self.write_capacity=1
        if (self.billing_mode == "PROVISIONED" and not self.read_capacity):
            self.read_capacity=1


@dataclass
class GlobalSecondaryIndex:
    hash_key: str
    name: str
    projection_type: str
    non_key_attributes: Optional[List[str]] = None
    range_key: Optional[str] = None
    read_capacity: Optional[int] = None
    write_capacity: Optional[int] = None

    def __post_init__(self):
        if self.projection_type == "INCLUDE" and not self.non_key_attributes:
            raise TheiaConfigurationError(
                "'non_key_attributes' needs to be set when using projection type - 'INCLUDE'"
            )

@dataclass
class LocalSecondaryIndex:
    name: str
    projection_type: str
    non_key_attributes: Optional[List[str]] = None
    range_key: str = None
    
@dataclass
class TableAttribute:
    name: str
    type: str
    

class Table(pulumi.ComponentResource):
    def __init__(
        self,
        resource_name: str,
        tableargs: Optional[TableSettings]=TableSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
        global_secondary_indexes: Optional[List[GlobalSecondaryIndex]]=None,
        local_secondary_indexes: Optional[List[LocalSecondaryIndex]]=None,
        ttl_attribute_name: Optional[str]=None,
        point_in_time_recovery_enabled: Optional[bool] = None,
        attributes: Optional[List[TableAttribute]]=None,
    ) -> None:
        super().__init__("theia:aws:database", resource_name, None, opts)

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        depends = []
        logger.info(f"creating dynamodb table resource {resource_name}")
        ttl_Args=None
        if (ttl_attribute_name):
            ttl_Args=dynamodb.TableTtlArgs(
                attribute_name=ttl_attribute_name,
                enabled=True,
            )
        if (point_in_time_recovery_enabled):
            point_in_time_recovery_e=dynamodb.TablePointInTimeRecoveryArgs(
                enabled=point_in_time_recovery_enabled,
            )

        attributes_args=[]
        if (attributes):
            for att in attributes:
                attributes_args.append(
                    dynamodb.TableAttributeArgs(
                        name=att.name,
                        type=att.type,
                    )
                )
        global_secondary_indexes_args=[]
        if (global_secondary_indexes):
            for global_indexes in global_secondary_indexes:
                global_secondary_indexes_args.append(
                    dynamodb.TableGlobalSecondaryIndexArgs(
                        hash_key=global_indexes.hash_key,
                        name=global_indexes.name,
                        projection_type=global_indexes.projection_type,
                        non_key_attributes=global_indexes.non_key_attributes,
                        range_key=global_indexes.range_key,
                        read_capacity=global_indexes.read_capacity,
                        write_capacity=global_indexes.write_capacity,
                    )
                )
        local_secondary_indexes_args=[]
        if (local_secondary_indexes):
            for local_indexes in local_secondary_indexes:
                local_secondary_indexes_args.append(
                    dynamodb.TableLocalSecondaryIndexArgs(
                        name=local_indexes.name,
                        projection_type=local_indexes.projection_type,
                        non_key_attributes=local_indexes.non_key_attributes,
                        range_key=local_indexes.range_key,
                    )
                )
        table_args = dynamodb.TableArgs(
            attributes=attributes_args,
            billing_mode=tableargs.billing_mode,
            deletion_protection_enabled=tableargs.deletion_protection_enabled,
            global_secondary_indexes=global_secondary_indexes_args,
            hash_key=tableargs.hash_key,
            local_secondary_indexes=local_secondary_indexes_args,
            name=resource_name,
            point_in_time_recovery=None if not point_in_time_recovery_enabled else point_in_time_recovery_e,
            range_key=tableargs.range_key,
            read_capacity=tableargs.read_capacity,
            replicas=None,
            restore_date_time=None,
            restore_source_name=None,
            restore_to_latest_time=None,
            server_side_encryption=None,
            stream_enabled=True if tableargs.stream_view_type else False,
            stream_view_type=tableargs.stream_view_type,
            table_class=tableargs.table_class,
            tags=merged_tags,
            ttl=ttl_Args,
            write_capacity=tableargs.write_capacity
        )
        self.dynamodb=dynamodb.Table(
            resource_name=f"{resource_name}",
            args=table_args,
            opts=ResourceOptions(parent=self, depends_on=depends),
        )
        pulumi.export(
            f"dynamodbTable---{resource_name}",
            {
                "id": self.dynamodb.id,
                "arn": self.dynamodb.arn,
                "stream_arn": self.dynamodb.stream_arn,
            },
        )

        self.register_outputs({})

    @staticmethod
    def import_(stack: str, resource_name: str) -> dynamodb.Table:
        stack_ref = StackRegistry.get(stack_name=stack)
        rds = stack_ref.get_output(name=f"dynamodbTable---{resource_name}")
        return dynamodb.Table.get(resource_name=resource_name, id=Table["id"])
