import pulumi
from pulumi_aws import sqs
from pulumi.output import Output
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, List, Mapping, Literal, Sequence
from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.utils.errors import TheiaConfigurationError

logger = Logger("sqs")

@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class QueueSettings:
    content_based_deduplication: Optional[bool] = None
    deduplication_scope: Optional[str] = None
    delay_seconds: Optional[int] = None
    fifo_queue: Optional[bool] = None
    fifo_throughput_limit: Optional[str] = None
    max_message_size: Optional[int] = None
    message_retention_seconds: Optional[int] = None
    policy: Optional[str] = None
    receive_wait_time_seconds: Optional[int] = None
    redrive_allow_policy: Optional[str] = None
    redrive_policy: Optional[Output[str] | str] = None
    sqs_managed_sse_enabled: Optional[bool] = None
    visibility_timeout_seconds: Optional[int] = None

class Queue():
    def __new__(
        cls,
        resource_name: str,
        queue_settings: Optional[QueueSettings]=QueueSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> sqs.QueueArgs:

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags
        if(queue_settings.fifo_queue == True):
            queue_name=f"{resource_name}.fifo"
        else:
            queue_name=f"{resource_name}"

        depends = []
        logger.info(f"creating SQS Queue resource {resource_name}")
        queue_args = sqs.QueueArgs(
            content_based_deduplication=queue_settings.content_based_deduplication,
            deduplication_scope=queue_settings.deduplication_scope,
            delay_seconds=queue_settings.delay_seconds,
            fifo_queue=queue_settings.fifo_queue,
            fifo_throughput_limit=queue_settings.fifo_throughput_limit,
            kms_data_key_reuse_period_seconds=None,
            kms_master_key_id=None,
            max_message_size=queue_settings.max_message_size,
            message_retention_seconds=queue_settings.message_retention_seconds,
            name_prefix=None,
            policy=queue_settings.policy,
            receive_wait_time_seconds=queue_settings.receive_wait_time_seconds,
            redrive_allow_policy=queue_settings.redrive_allow_policy,
            redrive_policy=queue_settings.redrive_policy,
            sqs_managed_sse_enabled=queue_settings.sqs_managed_sse_enabled,
            visibility_timeout_seconds=queue_settings.visibility_timeout_seconds,
            tags=merged_tags,
            name=queue_name,
        )
        sqs_out=sqs.Queue(
            resource_name=f"{resource_name}",
            args=queue_args,
            opts=opts,
        )
        pulumi.export(
            f"sqs---{resource_name}",
            {
                "id": sqs_out.id,
                "arn": sqs_out.arn,
                "url": sqs_out.url,
            },
        )
        return sqs_out
    @staticmethod
    def import_(stack: str, resource_name: str) -> sqs.Queue:
        stack_ref = StackRegistry.get(stack_name=stack)
        sqs = stack_ref.get_output(name=f"sqs---{resource_name}")
        return sqs.Queue.get(resource_name=resource_name, id=Queue["id"])
