import pulumi

from pulumi_aws import s3, iam, Provider, get_region
from pulumi import Output, InvokeOptions

from typing import Mapping, List, Optional, Literal
from pydantic.dataclasses import dataclass, ConfigDict

from theia import ResourceOptions
from theia.definitions import THEIA_VERSION
from theia._registry import StackRegistry, GlobalTags


@dataclass
class CorsRule:
    allowed_methods: List[Literal["GET", "POST", "PUT", "DELETE", "HEAD"]]
    allowed_origins: List[str]
    allowed_headers: Optional[List[str]] = None
    expose_headers: Optional[List[str]] = None
    max_age_seconds: Optional[int] = None


@dataclass
class LifecycleRule:
    name: str
    enabled: Optional[bool] = True
    expiration_days: Optional[int] = None
    nonconcurent_version_expiration: Optional[int] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class BucketObject:
    key: Optional[str] = None
    content: Optional[str | Output[str]] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class BucketSettings:
    acl: Optional[
        Literal[
            "private",
            "public-read",
            "public-read-write",
            "aws-exec-read",
            "authenticated-read",
            "bucket-owner-read",
            "bucket-owner-full-control",
        ]
    ] = "private"
    cors_rules: Optional[List[CorsRule]] = None
    hosted_zone_id: Optional[str | Output[str]] = None
    lifecycle_rules: Optional[List[LifecycleRule]] = None
    policy: Optional[dict] = None
    use_default_policy: Optional[bool] = None
    enable_versioning: Optional[bool] = False
    block_public_acls: Optional[bool] = False
    block_public_policy: Optional[bool] = False
    ignore_public_acls: Optional[bool] = False
    restrict_public_buckets: Optional[bool] = False
    apply_access_policy: Optional[bool] = None
    objects: Optional[List[BucketObject]] = None


class Bucket:
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[BucketSettings] = BucketSettings(),
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> s3.Bucket:
        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        if settings.cors_rules:
            bucket_cors = []
            for rule in settings.cors_rules:
                bucket_cors.append(
                    s3.BucketCorsRuleArgs(
                        allowed_headers=rule.allowed_headers,
                        allowed_methods=rule.allowed_methods,
                        allowed_origins=rule.allowed_origins,
                        expose_headers=rule.expose_headers,
                        max_age_seconds=rule.max_age_seconds,
                    )
                )

        if settings.lifecycle_rules:
            bucket_lifecycle_rules = []
            for rule in settings.lifecycle_rules:
                bucket_lifecycle_rules.append(
                    s3.BucketLifecycleRuleArgs(
                        enabled=rule.enabled,
                        expiration=s3.BucketLifecycleRuleExpirationArgs(days=rule.expiration_days),
                        noncurrent_version_expiration=s3.BucketLifecycleRuleNoncurrentVersionExpirationArgs(
                            days=rule.nonconcurent_version_expiration
                        ),
                        id=rule.name,
                    )
                )

        bucket_args = s3.BucketArgs(
            acl=settings.acl,
            cors_rules=None if not settings.cors_rules else bucket_cors,
            hosted_zone_id=settings.hosted_zone_id,
            lifecycle_rules=None if not settings.lifecycle_rules else bucket_lifecycle_rules,
            versioning=s3.BucketVersioningArgs(enabled=settings.enable_versioning),
            bucket=resource_name,
        )

        bucket = s3.Bucket(resource_name, args=bucket_args, opts=opts)

        if settings.use_default_policy:
            provider = Provider(
                f"{resource_name}-policy-provider",
                region=get_region().name,
                skip_metadata_api_check=False,
                skip_credentials_validation=True,
            )
            s3_policy = iam.get_policy_document_output(
                statements=[
                    iam.GetPolicyDocumentStatementArgs(
                        principals=[
                            iam.GetPolicyDocumentStatementPrincipalArgs(
                                type="*",
                                identifiers=["*"],
                            )
                        ],
                        actions=[
                            "s3:GetObject",
                        ],
                        resources=[f"arn:aws:s3:::{resource_name}/*"],
                    )
                ],
                opts=InvokeOptions(provider=provider),
            )

            _ = s3.BucketPolicy(
                resource_name + "-policy",
                bucket=bucket.id,
                policy=s3_policy.json,
                opts=ResourceOptions(parent=bucket, depends_on=bucket),
            )

        _ = s3.BucketPublicAccessBlock(
            resource_name=f"{resource_name}-block-public-access",
            bucket=bucket.id,
            block_public_acls=settings.block_public_acls,
            block_public_policy=settings.block_public_policy,
            restrict_public_buckets=settings.restrict_public_buckets,
            ignore_public_acls=settings.ignore_public_acls,
        )

        if settings.objects:
            for idx, object in enumerate(settings.objects):
                s3.BucketObject(
                    resource_name=f"{resource_name}-object-{idx}",
                    key=object.key,
                    content=object.content,
                    bucket=bucket.id,
                )

        pulumi.export(
            f"s3---{resource_name}",
            {
                "id": bucket.id,
                "bucket_domain": bucket.bucket_domain_name,
                "arn": bucket.arn,
                "name": bucket.bucket,
            },
        )

        return bucket

    @staticmethod
    def import_(stack: str, resource_name: str) -> s3.Bucket:
        stack_ref = StackRegistry.get(stack_name=stack)
        bucket = stack_ref.get_output(name=f"s3---{resource_name}")
        return s3.Bucket.get(resource_name=resource_name, id=bucket["id"])
