import logging

from pathlib import Path
from typing import Literal
from pydantic import BaseModel
from functools import lru_cache
from rich.logging import RichHandler


DATE_FORMAT = "%d %b %Y | %H:%M:%S"
LOGGER_FORMAT = "%(asctime)s | %(message)s"


class LoggerConfig(BaseModel):
    handlers: list
    format: str | None
    date_format: str | None = None
    logger_file: Path | None = None
    level: int = logging.DEBUG


@lru_cache
def get_logger_config():
    return LoggerConfig(
        handlers=[
            RichHandler(
                rich_tracebacks=False, tracebacks_show_locals=False, show_time=False
            ),
        ],
        format=None,
        date_format=DATE_FORMAT,
    )


def setup_rich_logger():
    for name in logging.root.manager.loggerDict.keys():
        logging.getLogger(name).handlers = []
        logging.getLogger(name).propagate = True

    logger_config = get_logger_config()

    logging.basicConfig(
        level=logger_config.level,
        format=logger_config.format,
        datefmt=logger_config.date_format,
        handlers=logger_config.handlers,
    )


class Logger:
    def __new__(
        cls, name: str, level: Literal["debug", "info", "warning", "error"] = "debug"
    ):
        # setup_rich_logger()
        logger = logging.getLogger(name)
        logger.setLevel(getattr(logging, level.upper()))

        return logger
